package piglets.beta;

import piglets.*;
import universum.bi.*;

/**
 * Created by IntelliJ IDEA.
 * User: Tanya
 * Date: 02.01.2007
 * Time: 21:47:09
 */
class SearchPath {
    public int      path_x;
    public boolean  explored;
    public Piglet   owner;

    public SearchPath(int path_x) {
        this.path_x = path_x;
    }
}

public class Herder extends SuperHerder {
    private SearchPath [] pathes;
    private boolean everythingExplored;

    public void init(UserGameInfo info) {
        super.init(info);
        pathes = new SearchPath[Constants.getWidth()];
        for (int i=0; i<Constants.getWidth(); ++i) {
            pathes[i] = new SearchPath(i);
        }
        everythingExplored = false;
    }

    protected void makeTurn(BeingInterface bi) {
        super.makeTurn(bi);

        for (SuperPiglet p : piglets) {
            Piglet piglet = (Piglet)p;

            Location pos = bi.getLocation(p);

            if (piglet.getState() == Piglet.State.UNKNOWN) {
                // try to find closest unexplored path for that piglet
                int posLeft     = pos.getX();
                int posRight    = Piglet.normalizeX(pos.getX() + 1);
                int foundPos    = 0;
                int foundNext   = 0;
                int foundPrev   = 0;

                int counter     = Constants.getWidth() / 2 + 1;

                while (counter > 0) {
                    if (pathes[posLeft].owner == null && !pathes[posLeft].explored) {
                        foundPos = posLeft;
                        foundNext = Piglet.normalizeX(foundPos - 1);
                        foundPrev = Piglet.normalizeX(foundPos + 1);
                        break;
                    }
                    if (pathes[posRight].owner == null && !pathes[posRight].explored) {
                        foundPos = posRight;
                        foundNext = Piglet.normalizeX(foundPos + 1);
                        foundPrev = Piglet.normalizeX(foundPos - 1);
                        break;
                    }
                    posLeft     = Piglet.normalizeX(posLeft - 1);
                    posRight    = Piglet.normalizeX(posRight + 1);
                    --counter;
                }

                if (counter > 0) {
                    if (pathes[foundNext].owner == null && !pathes[foundNext].explored
                    && (pathes[foundPrev].owner != null || pathes[foundPrev].explored)) {
                        foundPos = foundNext;
                    }

                    pathes[foundPos].owner = piglet;
                    pathes[Piglet.normalizeX(foundPos + 1)].owner = piglet;
                    pathes[Piglet.normalizeX(foundPos - 1)].owner = piglet;
                    piglet.setStateExplore();
                }
            } else if (everythingExplored && piglet.getState() != Piglet.State.GOBBLE) {
                // try to find draw-well for gobble
                DrawWell bestDrawWell = null;
                float maxWeight = -INF;

                for (DrawWell dw : drawWells) {
                    float weight = dw.getCell().growth * Piglet.R_wait / dw.owners.size() - Piglet.distance(pos, dw.pos) * piglet.getM() * Constants.K_movecost;
                    if (weight > maxWeight) {
                        maxWeight = weight;
                        bestDrawWell = dw;
                    }
                }

                if (bestDrawWell != null) {
                    bestDrawWell.owners.add(piglet);
                    piglet.setStateGobble(bestDrawWell);
                }
            }
        }
    }

    public synchronized void freePath(Piglet piglet) {
        for (int i=0; i<pathes.length; ++i) {
            if (pathes[i].owner == piglet) {
                pathes[i].owner = null;
                checkPathExplored(i);
            }
        }        
    }

    private void checkPathExplored(int i) {
        for (int j=0; j<Constants.getHeight(); ++j) {
            if (!glade[i][j].explored) {
                return ;
            }
        }
        pathes[i].explored = true;

        for (SearchPath p : pathes) {
            if (!p.explored) {
                return;
            }
        }
        everythingExplored = true;
    }
}
