package universum.bi;

import java.util.List;
import java.util.LinkedList;
import universum.engine.*;
/**
 *
 * @author nike
 */
public final class PointInfo {
    private Location loc;
    private Universe u;
    private List<Integer> ids;
    private Object mark;
    
    public PointInfo(Universe u, Location loc) {
        Constants.checkPerms();
        this.loc = loc;
        this.u = u;
        ids = new LinkedList<Integer>();
    }
    
    // location of this point
    public Location getLocation() {
        return loc;
    }
    
    // amount of energy at this point
    public float getCount(Being me) {
        // only known if close
        if (!u.checkDistance(me, getLocation())) {
            return -1f;
        }
        return u.getResourceControl().getCount(loc);
    }
    
    // how fast energy increases here
    public float getGrowthRate(Being me) {
        // always known 
        return u.getResourceControl().getGrowthRate(loc);
    }
    
    // the maximum amount of energy at the point
    public float getMaxCount(Being me) {
        // always known 
        return u.getResourceControl().getMaxCount(loc);
    }
    
    // returns ids of all entities  here, later can be used
    // as parameter for events and energy donation
    public synchronized Integer[] getEntities(Entity me, Integer[] data) {
        // only known if close
        if (!u.checkDistance(me, getLocation())) {
            return null;
        }
        int ilen = ids.size();
        if (data == null) {
            data = new Integer[ilen];
        }
        int dlen = data.length;
        data = ids.toArray(data);
        // null terminate, if needed
        if (dlen > ilen) {
            data[ilen] = null;
        }
        return data;
    }
   
    /*
    synchronized public void putMark(Object mark) {
        Constants.checkPerms();
        this.mark = mark;
    }

    synchronized public Object getMark() {
        return mark;
	} */

    // those two to not be used by beings
    synchronized public void addEntity(Integer id) {
        Constants.checkPerms();
        ids.add(id);
    }
    
    synchronized public void removeEntity(Integer id) {
        Constants.checkPerms();
        ids.remove(id);
    }

    synchronized public float totalMass(Being me) {
	if (me == null) {
	    // API only for the engine 
	    Constants.checkPerms();
	} else {
	    if (!u.checkDistance(me, getLocation())) {
		return -1f;
	    }
	}
	float rv = 0f;
	for (Integer i : ids) {
	    rv += u.getMass(i);
	}
	return rv;
    }
}
