package universum.engine;

import java.util.concurrent.*;
import java.util.*;

import universum.bi.*;
import universum.util.*;

class TurnService {
    class CustomThreadPoolExecutor extends ThreadPoolExecutor {
        CustomThreadPoolExecutor(int poolSize) {
            super(poolSize, poolSize,
                  0L, TimeUnit.MILLISECONDS,
                  new LinkedBlockingQueue<Runnable>());
        }
        
        public void afterExecute(Runnable r, Throwable t) {
            super.afterExecute(r, t);
            done();
        }
    }

    private final Universe u;
    private final CustomThreadPoolExecutor pool;
    int poolSize;

    TurnService(Universe u) {
        this.u = u;
        poolSize = 4; //Runtime.getRuntime().availableProcessors();
        // we create an additional thread to handle events fast
        // in case all other worker threads are busy        
        this.pool = new CustomThreadPoolExecutor(poolSize+1);
    }
   
    // maybe should be made atomic, but not that frequently used in fact
    volatile int active;

    // only single executeMulti allowed at the time
    void executeMulti(Runnable action) {
	if (pool.isShutdown()) {
            return;
        }
        synchronized (this) {
            active += poolSize;
        }
        for (int i=0; i<poolSize; i++) {
            pool.execute(action);
        }
    }

    Future<Object> executeSingle(Callable<Object> action) {
        if (pool.isShutdown()) {
            return null;
        }

        synchronized (this) {
            active++;
        }
        return pool.submit(action);
    }
   
    void waitCompletion() {
        for (int i = 0; ; i++) {
            if (i > 100) {
                // 2 secs
                u.checkHanging();
                // just for safety
                if (i > 1000) break;
            }
                 
            try {               
                synchronized (this) {
                    if (active == 0) {
                        break;
                    }
                    wait(20);
                }
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        }
    }

    synchronized void done() {
        active--;
        if (active == 0) {
            notify();
        }
    }

    void stop() {
        pool.shutdownNow();
    }
}
