package universum.ui;

import javax.swing.*;
import java.util.*;
import java.io.File;

import universum.bi.*;
import universum.engine.*;
import universum.util.*;

/**
 *
 * @author nike
 */
public class Controller extends GameUI implements GameOwner {
    private JDialog optDialog;
    private JDialog beingSelector;
    private Universe universe;
    ConfigFile cfgBeings;
    private boolean beingListChanged;
    private int playerCount;
    JFrame frame; 
    private GameInfo gi;    
    
    public Controller(JFrame owner, GameKind kind) {
        super(MODE_ADMINISTRATOR);

        this.frame = owner;
        cfgBeings = new ConfigFile("beings.properties");
        gi = new GameInfo();
        gi.gameKind = kind;

        beingListChanged = false;

        // gi can be updated by UI
        optDialog = new OptDialog(this, gi);
        beingSelector = new BeingSelectorDialog(this);

        startWorld();
    }

    public int getPlayerCount() {
        return universe == null ? 0 : universe.scores().size();
    }

    public ScoreData getPlayerScore(int playerId) {
        return universe.scores().get(universe.nameById(playerId));
    }

    protected void speedChanged(int v) {
        if (v > 0) {
            gi.turnDelay = 0;
            Constants.setRefreshTurns(v / 10 + 1);
        } else {
            gi.turnDelay = -v;
            Constants.setRefreshTurns(1);
        }
    }

    
    String getClassJar(String className, boolean canRebuild) {
        if (canRebuild) {
            String props = cfgBeings.getStrings(className, 2);
            if (!"".equals(props)) {
                System.out.println("Making being: "+className);
                universum.Main.makeBeing(props);
            }
        }
        
        String jar = cfgBeings.getStrings(className, 0);
        if (new File(jar).exists()) {
            return jar;
        }
        
        return universum.Main.beingPath(jar);
    }


    void addBeing(String className) {
        universe.addBeing(className, getClassJar(className, true));
        int nPlayers = universe.scores().size();
        if (nPlayers != playerCount) {
            beingListChanged = true;
            playerCount = nPlayers;
        }
    }

    void reinit() {
        universe = new Universe(this, gi);
        renderer = new Renderer(universe);
        playerCount = 0;
    }
    
    void startWorld() {	
        reinit();
        setPaused(true);
        universe.bigbang();        
    }
    
    void stopWorld() {
        if (universe != null) {            
            universe.apocalypse();
            universe = null;
        }
    }

    public synchronized void redraw() {
        redraw(beingListChanged);
        beingListChanged = false;
    }   

    public void notifyAboutCompletion(GameResult gr) {        
        JOptionPane.showMessageDialog(this,
                                      "Game finished!\n"+results(gr),
                                      "Finished",
                                      JOptionPane.INFORMATION_MESSAGE);      
        setPaused(true);
    }


    public void  setListenPort(int port) {
        // must be same as  asked
        assert port == 8889;
    }

    public int  getListenPort() {
        //return 0;
        return 8889;
    }

    public void setPaused(boolean paused) {
        universe.setPaused(paused);
        super.setPaused(paused);
        redraw();
    }

    public void buttonClick(Object source) {
        if (source == btnAdd) {
            beingSelector.setVisible(true);
            beingSelector.repaint();
        } else if (source == btnNew) {
            stopWorld();
            startWorld();
        } else if (source == btnOptions) {
            optDialog.setVisible(true);
            optDialog.repaint();
        } else {
            super.buttonClick(source);
        }
    }

}
