package universum.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;

import universum.bi.*;
import universum.engine.*;
import universum.util.Util;

import static universum.engine.RemoteProto.*;

/**
 *
 * @author pan
 */
public class GameUI extends JSplitPane 
    implements ActionListener, ChangeListener, RendererHolder {
    static final int MODE_OBSERVER      = 0;
    static final int MODE_ADMINISTRATOR = 1;
    static final int MODE_PLAYER = 2; 

    protected DrawSurface surface;
    protected JButton     btnAdd, btnNew, btnStart, btnOptions, btnAbout;
    protected JLabel      lblTurns;
    protected JSlider     speed;
    protected JTable      table;
    protected Renderer    renderer;
    protected Icon        startIcon, pauseIcon;
    private   boolean     paused;

    public GameUI(int mode) {
        super(HORIZONTAL_SPLIT);
        setLeftComponent(surface = new DrawSurface(this));
        setRightComponent(createToolBox(mode));
        setResizeWeight(1.0);
        startIcon = Images.getImageIcon("btn_start.png");
        pauseIcon = Images.getImageIcon("btn_pause.png");
        paused = true;
    }

    protected void speedChanged(int value) {
        // do nothing by default
    }

    Component createToolBox(int mode) {
        GridBagConstraints c = new GridBagConstraints();
        GridBagPanel toolbox = new GridBagPanel(c);

        c.insets = new Insets(2, 2, 2, 2);
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridwidth = GridBagConstraints.REMAINDER;
        toolbox.add(lblTurns = new JLabel("Turn: 0"));

        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1.0;
        c.weighty = 1.0;
        table = new JTable(new BeingTableModel());
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setShowVerticalLines(false);
        table.setRowHeight(20);
        table.setRowSelectionAllowed(false);
        int[] preferredWidth = new int[] {20, 100, 50, 50};
        TableColumnModel tcm = table.getColumnModel();
        int cols = tcm.getColumnCount();
        for (int i = 0; i < cols; i++) {
            tcm.getColumn(i).setPreferredWidth(preferredWidth[i]);
        }
        toolbox.add(new JScrollPane(table));
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weighty = 0.0;

        if (mode == MODE_ADMINISTRATOR) {
            c.gridwidth = 2;
            toolbox.add(btnAdd = new MyButton("Add beings", this));
            toolbox.add(btnNew = new MyButton("New game", this));
        }
        
        if (mode == MODE_ADMINISTRATOR || mode == MODE_PLAYER) {
            c.insets.top = 40;
            c.gridx = 0;
            c.gridwidth = GridBagConstraints.REMAINDER;
            speed = new JSlider(-500, 200, -Constants.getTurnDelay());
            Hashtable<Integer,JComponent> labels = new Hashtable<Integer,JComponent>();
            JLabel lblSlower = new JLabel("slower");
            JLabel lblFaster = new JLabel("faster");
            Font font = lblSlower.getFont();
            font = new Font(font.getName(), 0, font.getSize() - 1);
            lblSlower.setFont(font);
            lblFaster.setFont(font);
            labels.put(new Integer(-490), lblSlower);
            labels.put(new Integer(-150), new JLabel("Game Speed"));
            labels.put(new Integer(190), lblFaster);
            speed.setLabelTable(labels);
            speed.setMajorTickSpacing(50);
            speed.setPaintTicks(true);
            speed.setPaintLabels(true);
            speed.addChangeListener(this);
            toolbox.add(speed);
            c.gridx = GridBagConstraints.RELATIVE;
            c.insets.top = 40;
            c.gridwidth = 1;
            toolbox.add(btnStart = new MyButton("btn_start.png", "Start", this));
            c.insets.left = 10;
            c.insets.right = 5;
            toolbox.add(btnOptions = new MyButton("btn_options.png", "Options", this));
        }
        c.insets.left = 10;
        c.insets.right = 5;
        toolbox.add(btnAbout = new MyButton("btn_about.png", "About", this));

        toolbox.setMinimumSize(new Dimension(210, 0));
        return toolbox;
    }

    class BeingTableModel extends AbstractTableModel {
        String[] columnName = new String[] {
            "", "Owner", "Mass", "Energy"
        };

        public int getRowCount() {
            return getPlayerCount();
        }

        public int getColumnCount() {
            return columnName.length;
        }

        public String getColumnName(int col) {
            return columnName[col];
        }

        public Class getColumnClass(int col) {
            switch (col) {
            case 0:
                return ImageIcon.class;
            case 1:
                return String.class;
            default:
                return Integer.class;
            }
        }

        public Object getValueAt(int row, int col) {
            ScoreData sd = getPlayerScore(row);
            switch (col) {
            case 1:
                return sd.player;
            case 2:
                return (int)sd.score;
            case 3:
                return (int)sd.energy;
            default:
                return renderer.getPlayerIcon(sd.id);
            }
        }
    }

    public int getPlayerCount() {
        return 0;
    }

    public ScoreData getPlayerScore(int playerId) {
        return null;
    }

    public void setPaused(boolean paused) {
        btnStart.setIcon(paused ? startIcon : pauseIcon);
        btnStart.requestFocusInWindow();
        this.paused = paused;
    }

    public Renderer getRenderer() {
        return renderer;
    }
    
    public void redraw(boolean beingListChanged) {
        surface.redraw();
        if (beingListChanged) {
            table.revalidate();
        }
        table.repaint();
        RenderingInfo ri = renderer.ri;
        lblTurns.setText("Turn: " + ri.numTurns() + " of " + ri.maxTurns());
    }

    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if (source == speed) {
          int v = speed.getValue();
          speedChanged(speed.getValue());          
        }
    }

    private String javaInfo() {
        return 
            System.getProperty("java.vendor")+
            " "+
            System.getProperty("java.version");
    }

    public void showCredits() {
        final String credits = 
            "Java: " + javaInfo() + "\n" +
            "Engine: " + Constants.VERSION + "\n" +
            "Authors:\n" +
            "  Nikolay Igotti\n" + 
            "  Andrei Pangin\n" +
            "  Elena Kalmykova (artwork)\n"+
            "  others (see CREDITS)\n";
        JOptionPane.showMessageDialog(this,
                                      credits,
                                      "Electric Jungle's creators",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    public void buttonClick(Object source) {
        if (source == btnStart) {
            setPaused(!paused);
        } else if (source == btnAbout) {
            showCredits();
        }
    }

    public void actionPerformed(ActionEvent e) {
        buttonClick(e.getSource());
    }

    protected void showResults(RenderingInfo ri) {
        switch (ri.getStatus()) {
        case STATUS_COMPLETED: {
            // should pass GameKind
            GameResult gr = new GameResult(ri.scores(), ri.numTurns(), GameKind.JUNGLE);
            Util.messageBox( "Done",
                             "Game finished!\n"+results(gr));
            // we don't need to look at the game any more
            break;
        }
        case STATUS_FINISHED: {
            Util.messageBox( "Too late",
                             "Game already finished!\n");
            break;
        }
        }
    }

     protected String results(GameResult gr) {
        ScoreData winner = gr.winner();
        
        if (winner == null) {
            return "Noone survived";
        } else {
            return winner.player+" won with "+(int)winner.score+
                " points in "+gr.numTurns()+" turns"
                ;
        }
    }   
}
