package universum.ui;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;

import universum.engine.*;

class FileInfo extends RemoteInfo {
    ZipInputStream zis;
    ReadableByteChannel rbc;

    FileInfo(String file) throws IOException {
        super.init();
        zis = new ZipInputStream(new FileInputStream(file));
        zis.getNextEntry();
        rbc = Channels.newChannel(zis);
    }

   boolean refresh() throws IOException {
        doChannelRead(rbc);
        synchronized (this) {
            return parseBuf();
        }
    }   
}

public class LocalViewer extends GameUI {    
    private FileInfo fi;
    private boolean paused, needed;
    private int delay = 200;
    
    public LocalViewer(String file) throws IOException  {
        super(MODE_PLAYER);

        needed = true;
        fi = new FileInfo(file);
        renderer = new Renderer(fi);
        setPaused(false);

        new Thread(new Runnable() {
                public void run() {
                    while (needed) {
                        if (!paused) {
                            redraw();
                        }
                        try {
                            if (delay > 0) {
                                Thread.sleep(delay);
                            }
                        } catch (Exception e) {}
                    }
                }
            }).start();
    }
    
    

    public synchronized void redraw() {
        try {
            needed = fi.refresh();
            showResults(fi);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
        redraw(true);
    }

    public int getPlayerCount() {
        return fi == null ? 0 : fi.players;
    }

    public ScoreData getPlayerScore(int playerId) {
        return fi.scores().get(fi.nameById(playerId));
    }


    public void setPaused(boolean paused) {
        this.paused = paused;
        super.setPaused(paused);
    }

    
    protected void speedChanged(int v) {
        // XXX: rewrite me
        delay = 200 - v; 
        assert delay >= 0;
        redraw(true);
    }

}
